/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h>
#include <qtextstream.h>
#include <qtimer.h>

#include <kdebug.h>
#include <kstandarddirs.h>

#include "eventcontainer.h"

namespace EventWatcher {

EventContainer* eventContainer()
{
    static EventContainer *ec;
    if(ec == 0) {
        ec = new EventContainer();
    }
    return ec;
}

EventContainer::EventContainer(QObject *parent, const char *name)
 : QObject(parent, name)
{
    m_events.setAutoDelete(true);

    // load event history...
    load();

    // only save via a little timer: events can be grouped together when they are generated only a short time later...
    m_saveTimer = new QTimer(this);
    connect(m_saveTimer, SIGNAL(timeout()), this, SLOT(save()) );
}


EventContainer::~EventContainer()
{
}

const EventList EventContainer::getUnreadEvents() const
{
    // could probably be optimized with two separate lists for read and unread events...
    EventList match;
    for(EventList::Iterator it = m_events.begin(); it != m_events.end(); ++it) {
        if( (*it)->state == IsUnread )
            match.append( (*it) );
    }
    return match;
}

const EventList EventContainer::getReadEvents() const
{
    // could probably be optimized with two separate lists for read and unread events...
    EventList match;
    for(EventList::Iterator it = m_events.begin(); it != m_events.end(); ++it) {
        if( (*it)->state == IsRead )
            match.append( (*it) );
    }
    return match;
}

const EventList EventContainer::getAllEvents() const
{
    return m_events;
}

int EventContainer::countUnreadEvents()
{
    EventList events = getUnreadEvents();
    return events.count();
}

int EventContainer::countReadEvents()
{
    EventList events = getReadEvents();
    return events.count();
}

int EventContainer::countAllEvents()
{
    return m_events.count();
}

void EventContainer::load()
{
    QDomDocument doc( "EWEventML" );
    QFile file( locateLocal("appdata", "eventhistory.xml") );
    if( !file.open( IO_ReadOnly ) )
        return;
    if( !doc.setContent( &file ) ) {
        file.close();
        return;
    }
    file.close();

    QDomElement root = doc.documentElement();
    if( root.tagName() != "events" )
        return;

    QDomNode n = root.firstChild();
    while( !n.isNull() ) {
        QDomElement e = n.toElement();
        if( !e.isNull() ) {
            if( e.tagName() == "event" ) {
                EventItem *i = new EventItem();

                // id
                // find a decent default id...
                uint lastId, defaultId;
                if(m_events.last())
                    lastId = m_events.last()->id;
                else
                    lastId = 0;
                defaultId = lastId+1;
                i->id = e.attribute( "id", QString::number(defaultId) ).toUInt();

                // state
                QString state = e.attribute( "state", "IsRead" );
                if(state == "read")
                    i->state = IsRead;
                else
                    i->state = IsUnread;

                // nested elements...
                QString date, time, shortTitle, title, text, icon, actionText, actionData;
                ActionType actionType = NoAction;

                QDomNode n = e.firstChild();
                while( !n.isNull() ) {
                    QDomElement e = n.toElement();
                    if( !e.isNull() ) {
                        if( e.tagName() == "date" ) {
                            date = e.text();
                        } else if( e.tagName() == "time" ) {
                            time = e.text();
                        } else if( e.tagName() == "shortTitle" ) {
                            shortTitle = e.text();
                        } else if( e.tagName() == "title" ) {
                            title = e.text();
                        } else if( e.tagName() == "text" ) {
                            text = e.text();
                        } else if( e.tagName() == "icon" ) {
                            icon = e.text();
                        } else if( e.tagName() == "action" ) {
                            actionData = e.attribute( "data", "" );
                            actionText = e.attribute( "text", "..." );
                            QString type = e.attribute( "type", "NoAction" );
                            if(type == "UrlAction")
                                actionType = UrlAction;
                            else
                                actionType = NoAction;
                        }
                    }

                    n = n.nextSibling();
                }
                i->data = EventData(shortTitle, title, text, icon, EventAction(actionType, actionData, actionText) );
                i->data.setDate( QDate::fromString(date) );
                i->data.setTime( QTime::fromString(time) );
                m_events.append(i);
            }
        }

        n = n.nextSibling();
    }
}

void EventContainer::save()
{
    // write the data using Qt's dom classes...
    QDomDocument doc( "EWEventML" );

    QDomElement root = doc.createElement( "events" );
    doc.appendChild( root );

    for(EventList::Iterator it = m_events.begin(); it != m_events.end(); ++it) {
        root.appendChild( eventItemToNode(doc, *(*it)) );
    }

    QFile file( locateLocal("appdata", "eventhistory.xml") );
    if( !file.open(IO_WriteOnly) )
        return;

    QTextStream stream(&file);
    stream.setEncoding(QTextStream::UnicodeUTF8);

    stream << doc.toString();
}

QDomElement EventContainer::eventItemToNode( QDomDocument &d, const EventItem &i )
{
    QDomElement in = d.createElement( "event" );
 
    QDomAttr a = d.createAttribute( "id" );
    a.setValue( QString::number(i.id) );
    in.setAttributeNode( a );
 
    a = d.createAttribute( "state" );
    QString state;
    if(i.state == IsRead)
        state = "read";
    else
        state = "unread";
    a.setValue( state );
    in.setAttributeNode( a );

    QDomElement e = d.createElement( "date" );
    e.appendChild( d.createTextNode(i.data.date().toString()) );
    in.appendChild( e );

    e = d.createElement( "time" );
    e.appendChild( d.createTextNode(i.data.time().toString()) );
    in.appendChild( e );

    e = d.createElement( "shortTitle" );
    e.appendChild( d.createTextNode(i.data.shortTitle()) );
    in.appendChild( e );

    e = d.createElement( "title" );
    e.appendChild( d.createTextNode(i.data.title()) );
    in.appendChild( e );

    e = d.createElement( "text" );
    e.appendChild( d.createTextNode(i.data.text()) );
    in.appendChild( e );

    e = d.createElement( "icon" );
    e.appendChild( d.createTextNode(i.data.icon()) );
    in.appendChild( e );

    e = d.createElement( "action" );
    a = d.createAttribute( "type" );
    a.setValue( (i.data.action().actionType()==UrlAction)?"UrlAction":"NoAction" );
    e.setAttributeNode( a );
    a = d.createAttribute( "text" );
    a.setValue( i.data.action().actionText() );
    e.setAttributeNode( a );
    a = d.createAttribute( "data" );
    a.setValue( i.data.action().actionData() );
    e.setAttributeNode( a );
    in.appendChild( e );

    return in;
}

void EventContainer::addEvent(const EventData &data, const EventStates& state)
{
    EventItem *event = new EventItem;
    event->state = state;
    event->data = data;

    // id -- hope that procedure sufficient.
    uint lastId;
    if(m_events.last())
        lastId = m_events.last()->id;
    else
        lastId = 0;
    event->id = lastId+1;

    m_events.append( event );

    emit( eventListChanged() );
    emit( eventAdded(event) );

    // save to history...
    m_saveTimer->start(1000, true);
}

bool EventContainer::eventAction(uint eventId)
{
    // find event item with id eventId...
    for(EventList::Iterator it = m_events.begin(); it != m_events.end(); ++it) {
        if((*it)->id == eventId) {
            bool succ = (*it)->data.action().performAction();

            // save to history...
            m_saveTimer->start(1000, true);

            return succ;
        }
    }
    return false;
}

bool EventContainer::eventSetRead(uint eventId, bool read)
{
    // find event item with id eventId...
    for(EventList::Iterator it = m_events.begin(); it != m_events.end(); ++it) {
        if((*it)->id == eventId) {
            if(read)
                (*it)->state = IsRead;
            else
                (*it)->state = IsUnread;
            emit( eventListChanged() );
            emit( eventAdded( (*it) ) );

            // save to history...
            m_saveTimer->start(1000, true);

            return true;
        }
    }
    return false;
}

bool EventContainer::eventDelete(uint eventId)
{
    for(uint i=0; i < m_events.count(); ++i) {
        if(m_events.at(i)->id == eventId) {
            bool succ = m_events.remove(i);
            if(succ)
                emit( eventListChanged() );

            // save to history...
            m_saveTimer->start(1000, true);

            return succ;
        }
    }
    return false;

    // save to history...
    m_saveTimer->start(1000, true);
}

};
#include "eventcontainer.moc"
