/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfont.h>
#include <qimage.h>

#include <kdebug.h>
#include <kiconloader.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <kurl.h>

#include "eventcontainer.h"
#include "eventview.h"

namespace EventWatcher {

EventView::EventView(QWidget *parent, const char *name) : QScrollView( parent, name ),
  m_lastClickedItem(0)
{
    enableClipper( true );

    viewport()->setBackgroundMode( NoBackground );
};

void EventView::insertItem(EventViewItemBase* item)
{
    m_items.append(item);

    updateGeometry();
    updateContents();
}

void EventView::removeItem(EventViewItemBase* item)
{
    if( m_items.contains(item) ) {
        unselectItem(item);
        m_items.remove(item);
        delete item; // items are taking care for deleting theire child items themself.
    }

    updateGeometry();
    updateContents();
}

void EventView::clearItems()
{
    EventViewItemList::iterator it = m_items.begin();
    while(  it != m_items.end() ) {
        EventViewItemBase* item = (*it);
        it = m_items.remove( it );
        unselectItem( item );
        delete item; // items are taking care for deleting theire child items themself.
    }

    updateGeometry();
    updateContents();
}

EventViewItemList EventView::selectedItems()
{
    EventViewItemList selectedList;

    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        if( (*it)->selected() )
            selectedList.append( (*it) );
        // in case the item has childs, let it recursively add
        // selected childs to the list (passed by reference)
        if( (*it)->hasChildItems() && (*it)->showChildItems() )
            (*it)->selectedItemsRecursive(selectedList);
    }

    return selectedList;
}

EventViewItemBase* EventView::itemAt(const QPoint& pos)
{
    int py = pos.y();
    int h = 0;
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        int ih = (*it)->height();
        if(py >= h && py <= h+ih) {
            return (*it);
        }
        h += ih;
        // in case the item has childs, let it recursively check
        // if there is an item at that point
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            EventViewItemBase *item;
            item = (*it)->itemAtRecursive(pos, h);
            if( item != 0 )
                return item;
        }
    }
    return 0;
}

QRect EventView::itemGeometry(EventViewItemBase* item)
{
    int h = 0;
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        int ih = (*it)->height();
        if( (*it) == item ) {
            return QRect(0,h,(*it)->width(),ih);
        }
        h += ih;
        // in case the item has childs, let it recursively check
        // if the item can be found there etc.
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            QRect r;
            bool succ = (*it)->itemGeometryRecursive(r, item, h);
            if( succ )
                return r;
        }
    }
    return QRect();
}

void EventView::viewportResizeEvent(QResizeEvent *)
{
    updateGeometry();
}

void EventView::contentsMousePressEvent( QMouseEvent * e )
{
    if( e->button() & LeftButton ) {
        if ( !(e->state() & ControlButton) )
            unselectItems();

        m_mouseDown = true;
        EventViewItemBase *i = itemAt( e->pos() );
        if( i != 0 ) {
            if( (m_lastClickedItem != 0) && (e->state() & ShiftButton) ) {
                // toggle all between the old clicked and the current clicked item...
                toggleRange(m_lastClickedItem, i);
            } else {
                // toggle clicked item
                toggleItem( i );
            }
            m_lastClickedItem = i;
        }
    }
}

void EventView::contentsMouseReleaseEvent( QMouseEvent * e )
{
    if( e->button() & LeftButton ) {
        m_mouseDown = false;
    }
}

void EventView::contentsMouseMoveEvent( QMouseEvent * e )
{
    if( m_mouseDown ) {
        EventViewItemBase *i = itemAt( e->pos() );
        if( i != 0 ) {
            selectItem( i );
        }
    }

//     // TODO: scrolling...
}

void EventView::toggleItem(EventViewItemBase* i)
{
    if( !i->isSelectable() )
        return;

    if( !i->selected() ) {
        i->setSelected( true );
        ++m_selectedItems;
    } else {
        i->setSelected( false );
        --m_selectedItems;
    }
    repaintItem( i );
    emit( selectionChanged( selectedItems().count() /*m_selectedItems*/) );
}

void EventView::selectItem(EventViewItemBase* i)
{
    if( !i->isSelectable() )
        return;

    if( !i->selected() ) {
        i->setSelected( true );
        repaintItem( i );
        ++m_selectedItems;
        emit( selectionChanged( selectedItems().count() /*m_selectedItems*/) );
    }
}

void EventView::unselectItem(EventViewItemBase* i)
{
    if( !i->isSelectable() )
        return;

    if( i->selected() ) {
        i->setSelected( false );
        repaintItem( i );
        --m_selectedItems;
        emit( selectionChanged( selectedItems().count() /*m_selectedItems*/) );
    }
}

void EventView::toggleRange(EventViewItemBase* a, EventViewItemBase* b)
{
    bool startA = false,
         startB = false;
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        if( !startA && !startB ) {
            if( (*it) == a )
                startA = true;
            else if( (*it) == b )
                startB = true;
        }

        if( startA || startB ) {
            toggleItem( (*it) );

            if( (startA && (*it) == b) || (startB && (*it) == a) )
                break;
        }
    }
}

void EventView::selectItems()
{
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        selectItem( (*it) );
        // in case the item has childs, let it recursively select them...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->selectItemsRecursive();
        }
    }
}

void EventView::unselectItems()
{
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        unselectItem( (*it) );
        // in case the item has childs, let it recursively unselect them...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->unselectItemsRecursive();
        }
    }
}

void EventView::drawContents( QPainter *p, int clipx, int clipy, int clipw, int cliph )
{
    QRect cr(clipx,clipy,clipw,cliph);
    int h = 0;
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        int itemHeight = (*it)->height();
        QRect ir(0,h,visibleWidth(),itemHeight);
        if( ir.bottom() >= cr.top() && ir.top() <= cr.bottom() ) {
            QRect clip = cr, item = ir;
            QRect inter = cr&ir;
            inter.moveTop(inter.top()-h);

            m_bufferPixmap.resize(inter.width(), inter.height());
            m_bufferPainter.begin(&m_bufferPixmap);
            m_bufferPainter.save();
            m_bufferPainter.translate(-inter.x(), -inter.y());

            (*it)->drawContents(&m_bufferPainter, QPoint(0,0), inter );

            m_bufferPainter.restore();
            m_bufferPainter.end();

            if( p->device()->isExtDev() ) {
                p->drawPixmap(0+inter.x(),h+inter.y(),m_bufferPixmap/*, inter.x(),inter.y(),inter.width(),inter.height()*/);
            } else {
                bitBlt( p->device(), p->xForm( QPoint(0+inter.x(), h+inter.y()) ), &m_bufferPixmap/*, inter*/ );
            }
        }
        h += itemHeight;
        // in case the item has childs, let it recursively paint it's children...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->drawContentsRecursive(p, cr, h);
        }
    }

    // fill the empty area...
    if( h < visibleHeight() ) {
        QRect r = QRect( 0,h,visibleWidth(), visibleHeight()-h );
        p->fillRect( r, viewport()->colorGroup().base() );
    }
}

void EventView::repaintItem( EventViewItemBase* item )
{
    QRect r = itemGeometry( item );
    if( r.width() >= 1 && r.height() >= 1 )
        updateContents( r );
}

void EventView::updateGeometry()
{
    int w = visibleWidth();
    int h = 0;
    for( EventViewItemList::iterator it = m_items.begin(); it != m_items.end(); ++it ) {
        EventViewItemBase *item = (*it);
        item->setGeometry(w);

        QRect r = QRect(0,h,item->width(),item->height());
        updateItemWidgets( item, r );

        h += item->height();

        // in case the item has childs, let it recursively update their geometry...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->updateGeometryRecursive(w, h);
        }
        // update the widget visibility for all childs which are, _not_ visible.
        if( (*it)->hasChildItems() && !(*it)->showChildItems() ) {
            (*it)->showItemWidgetsRecursive( false );
        }
    }

    // make sure to set the area at least the size of the viewport so we have control over the
    // possible empty area (in case the item height is less than the viewport's) in drawContents.
    if( h > visibleHeight() ) {
        setVScrollBarMode( QScrollView::AlwaysOn );
        resizeContents(w,h);
    } else {
        setVScrollBarMode( QScrollView::AlwaysOff );
        resizeContents( w, visibleHeight() );
    }
}

void EventView::updateItemWidgets(EventViewItemBase* item, QRect r)
{
    bool showItem = true;
    if( item->parentItem() && !item->parentItem()->showChildItems() )
        showItem = false;

    // align widgets and/or show/hide if necessary...
    if( item->hasActionWidget() && item->actionWidget() ) {
        if( showItem ) {
            QRect ra = item->actionWidgetGeometry();
            moveChild( item->actionWidget(), r.x()+ra.x(), r.y()+ra.y() );
            // show if necessary...
            if( item->actionWidget()->isHidden() )
                item->actionWidget()->show();
        } else {
            // hide if necessary...
            if( !item->actionWidget()->isHidden() )
                item->actionWidget()->hide();
        }
    }
    if( item->hasControlWidget() && item->controlWidget() ) {
        if( showItem ) {
            QRect rc = item->controlWidgetGeometry();
            moveChild( item->controlWidget(), r.x()+rc.x(), r.y()+rc.y() );
            // show if necessary...
            if( item->controlWidget()->isHidden() )
                item->controlWidget()->show();
        } else {
            // hide if necessary...
            if( !item->controlWidget()->isHidden() )
                item->controlWidget()->hide();
        }

        // If also has keep button
        if( item->hasKeepWidget() && item->keepWidget() ) {
            if( showItem ) {
                QRect rc = item->controlWidgetGeometry();
                QRect rk = item->keepWidgetGeometry();
                moveChild( item->keepWidget(), r.x()+rk.x()+rc.width(), r.y()+rk.y() );
                // show if necessary...
                if( item->keepWidget()->isHidden() )
                    item->keepWidget()->show();
            } else {
                // hide if necessary...
                if( !item->keepWidget()->isHidden() )
                    item->keepWidget()->hide();
            }
        }
    }

}

bool EventView::loadMimeSource(const QString &source)
{
    // already loaded
    if(m_mimeSourceMap[source] == true)
        return true;

    // check if the source is a valid URL
    if(KURL(source).isValid()) {
        KURL url(source);

        // download if necessary
        QString filename;
        if ( url.isLocalFile() )
            filename = url.path();
        else
        {
            if ( !KIO::NetAccess::download( url, filename, NULL ) )
            {
                kdWarning() << i18n( "An error occured while downloading the file \"%1\"!" ).arg(url.prettyURL());
                return false;
            }
        }

        QImage img(filename);

        mimeSourceFactory()->setImage(source, img);
        m_mimeSourceMap[source] = true;
    } else
    // else it's probably of a local icon...
    {
        mimeSourceFactory()->setPixmap( source,
                KGlobal::iconLoader()->loadIcon(source, KIcon::NoGroup, 16) );
        m_mimeSourceMap[source] = true;
    }

    return true;
}

EventViewItemBase::EventViewItemBase(EventView *ev)
  : m_ev(ev),
    m_parentItem(0),
    m_showChildItems(false),
    m_selected(false),
    m_actionWidget(0),
    m_controlWidget(0)
{
}

EventViewItemBase::~EventViewItemBase()
{
    // delete child items.
    EventViewItemList::iterator it = m_childList.begin();
    while(  it != m_childList.end() ) {
        EventViewItemBase* item = (*it);
        it = m_childList.remove( it );
        eventView()->unselectItem( item );
        delete item; // items are taking care for deleting theire child items themself.
    }
}

EventViewItemBase* EventViewItemBase::parentItem()
{
    return m_parentItem;
}

void EventViewItemBase::setParentItem(EventViewItemBase* parent)
{
    m_parentItem = parent;
}

bool EventViewItemBase::hasChildItems() const
{
    return (m_childList.count()>=1);
}

void EventViewItemBase::insertChildItem(EventViewItemBase* item)
{
    item->setParentItem(this);
    m_childList.append(item);
//     showItemWidgetsRecursive( showChildItems() );

    // repaint the whole view
    eventView()->updateGeometry();
    eventView()->updateContents();
}

void EventViewItemBase::removeChildItem(EventViewItemBase* item)
{
    if( m_childList.contains(item) ) {
        eventView()->unselectItem(item);
        m_childList.remove(item);
        delete item; // items are taking care for deleting their child items themself.
    }

    eventView()->updateGeometry();
    eventView()->updateContents();
}

const EventViewItemList& EventViewItemBase::childItems() const
{
    return m_childList;
}

void EventViewItemBase::setShowChildItems(bool show)
{
    m_showChildItems = show;
    // hide _all_ child widgets when switching them to invisible.
    if( !show )
        showItemWidgetsRecursive(false);

    // repaint the whole view
    eventView()->updateGeometry();
    eventView()->updateContents();
}

bool EventViewItemBase::showChildItems() const
{
    return m_showChildItems;
}

void EventViewItemBase::selectedItemsRecursive(EventViewItemList &list)
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        if( (*it)->selected() )
            list.append( (*it) );
        // in case the (child-)item has childs, let it recursively add
        // selected childs to the list (passed by reference)
        if( (*it)->hasChildItems() && (*it)->showChildItems() )
            (*it)->selectedItemsRecursive(list);
    }
}

EventViewItemBase* EventViewItemBase::itemAtRecursive(const QPoint& pos, int &h)
{
    int py = pos.y();
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        int ih = (*it)->height();
        if(py >= h && py <= h+ih) {
            return (*it);
        }
        h += ih;
        // in case the (child-)item has childs, let it recursively check
        // if there is an item at that point
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            EventViewItemBase *item;
            item = (*it)->itemAtRecursive(pos, h);
            if( item != 0 )
                return item;
        }
    }
    // nothing has been found... return 0.
    return 0;
}

bool EventViewItemBase::itemGeometryRecursive(QRect &r, EventViewItemBase* item, int &h)
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        int ih = (*it)->height();
        if( (*it) == item ) {
            r = QRect(0,h,(*it)->width(),ih);
            return true;
        }
        h += ih;
        // in case the item has childs, let it recursively check
        // if the item can be found there etc.
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            bool succ = (*it)->itemGeometryRecursive(r, item, h);
            if( succ )
                return true;
        }
    }
    return false;
}

void EventViewItemBase::selectItemsRecursive()
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        eventView()->selectItem( (*it) );
        // in case the item has childs, let it recursively select them...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->selectItemsRecursive();
        }
    }
}

void EventViewItemBase::unselectItemsRecursive()
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        eventView()->unselectItem( (*it) );
        // in case the item has childs, let it recursively unselect them...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->unselectItemsRecursive();
        }
    }
}

void EventViewItemBase::drawContentsRecursive(QPainter *p, const QRect& cr, int &h)
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        int itemHeight = (*it)->height();
        QRect ir(0,h,eventView()->visibleWidth(),itemHeight);
        if( ir.bottom() >= cr.top() && ir.top() <= cr.bottom() ) {
            QRect clip = cr, item = ir;
            QRect inter = cr&ir;
            inter.moveTop(inter.top()-h);

            eventView()->m_bufferPixmap.resize(inter.width(), inter.height());
            eventView()->m_bufferPainter.begin(&(eventView()->m_bufferPixmap));
            eventView()->m_bufferPainter.save();
            eventView()->m_bufferPainter.translate(-inter.x(), -inter.y());

            (*it)->drawContents(&(eventView()->m_bufferPainter), QPoint(0,0), inter );

            eventView()->m_bufferPainter.restore();
            eventView()->m_bufferPainter.end();

            if( p->device()->isExtDev() ) {
                p->drawPixmap(0+inter.x(),h+inter.y(),eventView()->m_bufferPixmap );
            } else {
                bitBlt( p->device(), p->xForm( QPoint(0+inter.x(), h+inter.y()) ), &(eventView()->m_bufferPixmap) );
            }
        }
        h += itemHeight;
        // in case the item has childs, let it recursively paint it's children...
        if( (*it)->hasChildItems() && (*it)->showChildItems() ) {
            (*it)->drawContentsRecursive(p, cr, h);
        }
    }
}

void EventViewItemBase::updateGeometryRecursive(int w, int &h)
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        EventViewItemBase *item = (*it);
        item->setGeometry(w);

        // align widgets...
        QRect r = QRect(0,h,item->width(),item->height());
        eventView()->updateItemWidgets( item, r );

        h += item->height();

        // in case the item has childs, let it recursively update their geometry...
        if( item->hasChildItems() && item->showChildItems() ) {
            item->updateGeometryRecursive(w, h);
        }
    }
}

void EventViewItemBase::showItemWidgetsRecursive(bool show)
{
    for( EventViewItemList::iterator it = m_childList.begin(); it != m_childList.end(); ++it ) {
        if( (*it)->hasActionWidget() && (*it)->actionWidget() ) {
            if( show ) {
                if( (*it)->actionWidget()->isHidden() )
                    (*it)->actionWidget()->show();
            } else {
                // hide if necessary...
                if( !(*it)->actionWidget()->isHidden() )
                    (*it)->actionWidget()->hide();
            }
        }
        if( (*it)->hasControlWidget() && (*it)->controlWidget() ) {
            if( show ) {
                if( (*it)->controlWidget()->isHidden() )
                    (*it)->controlWidget()->show();
            } else {
                // hide if necessary...
                if( !(*it)->controlWidget()->isHidden() )
                    (*it)->controlWidget()->hide();
            }
        }
		if( (*it)->hasKeepWidget() && (*it)->keepWidget() ) {
            if( show ) {
                if( (*it)->keepWidget()->isHidden() )
                    (*it)->keepWidget()->show();
            } else {
                // hide if necessary...
                if( !(*it)->keepWidget()->isHidden() )
                    (*it)->keepWidget()->hide();
			}
		}
        // in case the item has childs, let it recursively update their widgets too...
        if( (*it)->hasChildItems() ) {
            (*it)->showItemWidgetsRecursive(show);
        }
    }
}

};

#include "eventview.moc"
