/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>

#include <kaction.h>
#include <kactionclasses.h>
#include <kapplication.h>
#include <kedittoolbar.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <kpopupmenu.h>
#include <kshortcut.h>
#include <kstdaction.h>
#include <kwin.h>

#include "config.h"
#include "eventwatcher.h"
#include "eventcontainer.h"
#include "pluginmanager.h"
#include "settingsdialog.h"
#include "systemtray.h"
#include "lib/plugin.h"
#include "lib/eventdata.h"
#include "mainwidget.h"
#include "popupmanager.h"

namespace EventWatcher {

eventwatcher::eventwatcher()
    : KMainWindow( 0, "eventwatcher" ),m_shuttingDown(false)
{
    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    // holds the list of recent events... mainWidget.
    m_mainWidget = new MainWidget(this, "mainWidget");
    setCentralWidget(m_mainWidget);

    setupSystemTray();

    // initialize EventContainer which will receive events, store them...
    m_eventContainer = eventContainer();

    // holds and manages the EventWidgets showing up...
    m_popupManager = new PopupManager(m_mainWidget);
    KWin::setOnAllDesktops(m_popupManager->winId() , true);
    m_popupManager->setAnchor( m_systemTray->mapToGlobal(QPoint(m_systemTray->width()/2, m_systemTray->height()/2)) );
    connect( m_popupManager, SIGNAL(restoreMainWindow()), this, SLOT(minimizeRestore()) );
    connect( m_eventContainer, SIGNAL(eventAdded(EventItem*)), m_popupManager, SLOT(slotEventAdded(EventItem*)) );

    connect( m_eventContainer, SIGNAL(eventAdded(EventItem*)), m_mainWidget, SLOT(slotEventAdded(EventItem*)) );
    connect( m_eventContainer, SIGNAL(eventListChanged()), m_systemTray, SLOT(slotEventListChanged()) );

    // create PluginManager singleton and connect...
    PluginManager *pluginMgr = pluginManager();
    connect( this, SIGNAL(settingsChanged()), pluginMgr, SLOT(settingsChanged()) );
    connect( pluginMgr, SIGNAL(pluginCreated(Plugin*)), this, SLOT(setupPlugin(Plugin*)) );
    pluginMgr->loadEnabledPlugins();

    // set the shell's ui resource file
    setXMLFile("eventwatcherui.rc");
    // create actions and add some of them to the system tray context menu, too
    setupActions();
    createGUI();
}

eventwatcher::~eventwatcher()
{
}

void eventwatcher::setupActions()
{
    KStdAction::quit( this, SLOT( slotQuit() ), actionCollection() );

    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());

    KAction *prefs = KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
    // add to system tray too.
    KPopupMenu *m = m_systemTray->contextMenu();
    connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(slotQuit()));
    prefs->plug( m, 1 );

    KAction *copy = KStdAction::copy( m_mainWidget, SLOT( slotCopy() ), actionCollection() );
    connect( m_mainWidget, SIGNAL(itemsSelected(bool)), copy, SLOT(setEnabled(bool)) );

    KStdAction::selectAll( m_mainWidget, SLOT( slotSelectAll() ), actionCollection() );

    KAction *del = new KAction(i18n("&Delete"), "editdelete", Key_Delete,
                                    m_mainWidget, SLOT(slotDelete()), actionCollection(), "delete_selected");
    connect( m_mainWidget, SIGNAL(itemsSelected(bool)), del, SLOT(setEnabled(bool)) );

    KAction *delAll = new KAction(i18n("Delete &All"), "editdelete", 0,
                                    m_mainWidget, SLOT(slotDelete()), actionCollection(), "delete_all_items");
    connect( m_mainWidget, SIGNAL(itemsSelected(bool)), delAll, SLOT(setEnabled(bool)) );

    KAction *store = new KAction(i18n("&Store"), "folder_favorite", 0,
                                    m_mainWidget, SLOT(slotKeep()), actionCollection(), "store_selected");
    connect( m_mainWidget, SIGNAL(itemsSelected(bool)), store, SLOT(setEnabled(bool)) );

    KToggleAction *group = new KToggleAction(i18n("&Group Events"), "view_tree", 0,
                                    actionCollection(), "group_similar_events");
    group->setChecked( Config::eventViewGroupItems() );
    connect( group, SIGNAL(toggled(bool)), m_mainWidget, SLOT(slotSetGroupItems(bool)) );

    KToggleAction *showDel = new KToggleAction(i18n("Show &Delete Buttons"), QString::null, 0,
                                    actionCollection(), "show_delete_buttons");
    showDel->setChecked( Config::eventViewDeleteButtons() );
    connect( showDel, SIGNAL(toggled(bool)), m_mainWidget, SLOT(slotSetShowDeleteButtons(bool)) );

    KToggleAction *smallBtn = new KToggleAction(i18n("&Small Action Buttons"), QString::null, 0,
                                    actionCollection(), "small_action_buttons");
    smallBtn->setChecked( Config::eventViewSmallActionButtons() );
    connect( smallBtn, SIGNAL(toggled(bool)), m_mainWidget, SLOT(slotSetSmallActionButtons(bool)) );
}

void eventwatcher::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());

    KEditToolbar dlg(actionCollection());
    connect(&dlg, SIGNAL( newToolbarConfig() ), this, SLOT( newToolbarConfig() ));
    dlg.exec();
}

// don't close the window when clicking close-button, example taken from JuK's sources - tpr @ 20040225
bool eventwatcher::queryClose()
{
    if(!m_shuttingDown) {
        hide();
        return false;
    }
    else return true;
}

void eventwatcher::setupSystemTray()
{
    m_systemTray = new SystemTray(this);
    m_systemTray->show();
}

void eventwatcher::setupPlugin(Plugin* p)
{
    // a new plugin has been created, some objects might be interested in its events...
    connect( p, SIGNAL(event(const EventData& )), this, SLOT(eventPopup(const EventData& )) );
    connect( p, SIGNAL(event(const EventData& )),
            m_eventContainer, SLOT(addEvent(const EventData& )) );
    connect( this, SIGNAL(settingsChanged()), p, SLOT(settingsChanged()) );
}

void eventwatcher::eventPopup(const EventData&)
{
    // currently only used to set the anchor for the popup...
    m_popupManager->setAnchor( m_systemTray->mapToGlobal(QPoint(m_systemTray->width()/2, m_systemTray->height()/2)) );
}

void eventwatcher::minimizeRestore(bool restore)
{
    KWin::WindowInfo info = KWin::windowInfo( winId(), NET::WMGeometry | NET::WMDesktop );
    if ( restore )
    {
        // set to the current desktop if necessary
        KWin::setOnDesktop( winId(), KWin::currentDesktop());
        // activate
        show();
        raise();
        KWin::activateWindow( winId() );
    } else {
        hide();
    }
}

void eventwatcher::optionsPreferences()
{
    SettingsDialog settings(this);
    settings.setInitialSize(QSize(600,400));

    connect( &settings, SIGNAL(settingsChanged()), this, SIGNAL(settingsChanged()) );

    if (settings.exec()==QDialog::Accepted)
    {
        settings.updateConfiguration();
        emit( settingsChanged() );
    }
}

};

#include "eventwatcher.moc"
