/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qclipboard.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qtoolbutton.h>
#include <qtooltip.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>

#include "plusminusbutton.h"
#include "lib/eventdata.h"
#include "eventcontainer.h"
#include "config.h"
#include "embed.h"
#include "mainwidget.h"

namespace EventWatcher {

MainWidget::MainWidget(QWidget *parent, const char *name)
 : QWidget(parent, name), m_oldHeader(0), m_newHeader(0)
{
    QVBoxLayout *l = new QVBoxLayout(this);
    m_eventView = new EventView(this, "new_event_view");
    l->addWidget( m_eventView );
    connect( m_eventView, SIGNAL(selectionChanged(int)), this, SLOT(slotSelectionChanged(int)) );

    // add all currently available events...
    slotUpdate();
}


MainWidget::~MainWidget()
{
}

void MainWidget::slotSelectionChanged(int selected)
{
    emit(itemsSelected(selected));
}

QString MainWidget::groupKey(const EventItem* event) const
{
    QString key = event->data.icon() + "_" + event->data.title(); // key consists of icon and title...
    if( event->state == IsUnread )
        key += "_unread";
    else
        key += "_read";
    return key;
}

void MainWidget::slotEventAdded(EventItem* event)
{
    if( !Config::eventViewGroupItems() ) {
        // just add them as children of m_newHeader.
        EventViewItem *item = new EventViewItem(Config::eventViewDeleteButtons(), event->state == IsUnread, Config::eventViewSmallActionButtons(), m_eventView);
        connect( item, SIGNAL(deleteItem(EventViewItem*)), this, SLOT(slotDeleteItem(EventViewItem*)) );
        connect( item, SIGNAL(keepItem(EventViewItem*)), this, SLOT(slotKeepItem(EventViewItem*)) );
        item->setEventItem( event );
        if( event->state == IsUnread )
            m_newHeader->insertChildItem( item );
        else
            m_oldHeader->insertChildItem( item );
    } else {
        // see if it fits into an existing group and add as child of it. if it
        // doesn't, also create a new group.
        QString key = groupKey( event );
        if( !m_groupMap.contains( key ) ) {
            EventViewGroupItem *group = new EventViewGroupItem(m_eventView);
            m_groupMap[key] = group;
            group->setIcon( event->data.icon() );
            group->setText( event->data.title() );
            group->setShowChildItems(true);
            group->setDisplayChildNumber(true);
            if( event->state == IsUnread )
                m_newHeader->insertChildItem( group );
            else
                m_oldHeader->insertChildItem( group );
        }
        EventViewItem *item = new EventViewItem(Config::eventViewDeleteButtons(), event->state == IsUnread, Config::eventViewSmallActionButtons(), m_eventView);
        connect( item, SIGNAL(deleteItem(EventViewItem*)), this, SLOT(slotDeleteItem(EventViewItem*)) );
        connect( item, SIGNAL(keepItem(EventViewItem*)), this, SLOT(slotKeepItem(EventViewItem*)) );
        item->setGrouped( true );
        item->setMarginLeft( 25 );
        item->setEventItem( event );
        m_groupMap[key]->insertChildItem( item );
    }
}

void MainWidget::slotUpdate()
{
    m_eventView->clearItems();
    m_groupMap.clear();

    m_oldHeader = new EventViewHeaderItem(m_eventView);
    m_oldHeader->setText( i18n("List of Old Events") );
    m_oldHeader->setShowChildItems(false);
    m_oldHeader->setDisplayChildNumber(!Config::eventViewGroupItems() );
    m_eventView->insertItem( m_oldHeader );

    m_newHeader = new EventViewHeaderItem(m_eventView);
    m_newHeader->setText( i18n("List of New Events") );
    m_newHeader->setShowChildItems(true);
    m_newHeader->setDisplayChildNumber(!Config::eventViewGroupItems() ); // only if events aren't grouped together in "sub-headers"...
    m_eventView->insertItem( m_newHeader );

    EventList events = eventContainer()->getAllEvents();
    for(EventList::Iterator it = events.begin(); it != events.end(); ++it) {
        slotEventAdded( *it );
    }
}

void MainWidget::slotCopy()
{
    EventViewItemList items = m_eventView->selectedItems();
    QString copyText;
    for(EventViewItemList::iterator it = items.begin(); it != items.end(); ++it) {
        EventViewItem* item = dynamic_cast<EventViewItem*>(*it);
        if( item ) {
            if( !copyText.isEmpty() )
                copyText += '\n';
            copyText += item->eventItem()->data.title() +'\n';
            copyText += item->eventItem()->data.text() + '\n';
        }
    }

    QClipboard *cb = QApplication::clipboard();
    // Copy text into the clipboard
    cb->setText( copyText,
                 QClipboard::Clipboard );
}

void MainWidget::slotDelete()
{
    EventViewItemList items = m_eventView->selectedItems();
    for(EventViewItemList::iterator it = items.begin(); it != items.end(); ++it) {
        EventViewItem* item = dynamic_cast<EventViewItem*>(*it);
        if( item ) {
            slotDeleteItem( item );
        }
    }
}

void MainWidget::slotKeep()
{
    EventViewItemList items = m_eventView->selectedItems();
    for(EventViewItemList::iterator it = items.begin(); it != items.end(); ++it) {
        EventViewItem* item = dynamic_cast<EventViewItem*>(*it);
        if( item ) {
            slotKeepItem( item );
        }
    }
}

void MainWidget::slotSelectAll()
{
    m_eventView->selectItems();
}

void MainWidget::slotSetGroupItems(bool group)
{
    Config::setEventViewGroupItems(group);
    Config::writeConfig();
    // rebuild the whole view
    slotUpdate();
}

void MainWidget::slotSetShowDeleteButtons(bool show)
{
    Config::setEventViewDeleteButtons(show);
    Config::writeConfig();
    // rebuild the whole view
    slotUpdate();
}

void MainWidget::slotSetSmallActionButtons(bool small)
{
    Config::setEventViewSmallActionButtons(small);
    Config::writeConfig();
    // rebuild the whole view
    slotUpdate();
}

void MainWidget::slotDeleteItem(EventViewItem* item)
{
    // delete...
    uint id = item->eventItem()->id;
    // also keep track of group items and delete it (which includes the last
    // child to be deleted too), in case it has only one or less more children
    QString key = groupKey( item->eventItem() );
    if( Config::eventViewGroupItems() && m_groupMap.contains(key) && (m_groupMap[key]->childItems().count() <= 1) ) {
        EventViewGroupItem* groupItem = m_groupMap[key];
        if( groupItem->parentItem() )
            groupItem->parentItem()->removeChildItem( groupItem );
        else
            m_eventView->removeItem( groupItem );
        m_groupMap.remove( key );
    } else {
        // delete from the list (will be announced as added by the conainer) and set read...
        if( item->parentItem() )
            item->parentItem()->removeChildItem( item ); // ugh. :)
        else // should be in the main event view then...
            m_eventView->removeItem( item );
    }
    eventContainer()->eventDelete( id );
}

void MainWidget::slotKeepItem(EventViewItem* item)
{
    const EventItem* eventItem = item->eventItem();
    bool isUnread = eventItem->state == IsUnread;
    uint id = eventItem->id;
    // also keep track of group items and delete it (which includes the last
    // child to be deleted too), in case it has only one or less more children
    QString key = groupKey( eventItem );
    if( Config::eventViewGroupItems() && m_groupMap.contains(key) && (m_groupMap[key]->childItems().count() <= 1) ) {
        EventViewGroupItem* groupItem = m_groupMap[key];
        if( groupItem->parentItem() )
            groupItem->parentItem()->removeChildItem( groupItem );
        else
            m_eventView->removeItem( groupItem );
            m_groupMap.remove( key );
    } else {
        // delete from the list (will be announced as added by the conainer) and set read...
        if( item->parentItem() )
            item->parentItem()->removeChildItem( item ); // ugh. :)
        else // should be in the main event view then...
            m_eventView->removeItem( item );
        }
    eventContainer()->eventSetRead( id, isUnread );
}

EventActionButton::EventActionButton(uint eventId, EventAction action, QWidget *parent, const char *name)
 : KPushButton(parent,name),
   m_eventId(eventId),
   m_action(action)
{
    connect(this, SIGNAL(clicked()), this, SLOT(buttonClicked()));
    setDragEnabled(true);
}

EventActionButton::~EventActionButton()
{
}

QDragObject * EventActionButton::dragObject()
{
    if(m_action.actionType() == UrlAction) {
        QStrList uri;
        uri.append(m_action.actionData().local8Bit());
        return new QUriDrag(uri, this);
    } else
        return new QTextDrag(m_action.actionData(), this);
}

void EventActionButton::buttonClicked()
{
//     if( m_hasAction && m_eventItem ) {
        eventContainer()->eventAction( m_eventId );
//     }
}


EventViewItem::EventViewItem(bool showDelete, bool showKeep, bool smallActionButton, EventView *ev) : EventViewItemBase(ev),
  m_grouped(false),
  m_rt(0),
  m_eventItem(0),
  m_hasAction(false),
  m_hasControlDelete(showDelete),
  m_hasControlKeep(showKeep),
  m_smallActionButton(smallActionButton)
{
    // Create close button
    if(m_hasControlDelete) {
        m_button = new QToolButton(eventView()->viewport(), "close_button");
        eventView()->addChild( m_button );
        setControlWidget(m_button);
        QPixmap pm(uic_findImage("deleteevent.png"));
        m_button->setPixmap( pm );
        QToolTip::add(m_button, i18n("Delete event") );
        m_button->resize( pm.width()+4, pm.height()+4 );
        connect( m_button, SIGNAL(clicked()), this, SLOT(slotControlClicked()) );
    }

    // Create keep button
    if(m_hasControlKeep) {
        m_button = new QToolButton(eventView()->viewport(), "keep_button");
        eventView()->addChild( m_button );
        setKeepWidget(m_button);
        QPixmap pm(uic_findImage("keepevent.png"));
        m_button->setPixmap( pm );
        QToolTip::add(m_button, i18n("Keep event") );
        m_button->resize( pm.width()+4, pm.height()+4 );
        connect( m_button, SIGNAL(clicked()), this, SLOT(slotKeepClicked()) );
    }

    setMarginLeft( 5 );
}

EventViewItem::~EventViewItem()
{
    delete m_rt;
    if( actionWidget() ) {
        eventView()->removeChild( actionWidget() );
        delete actionWidget();
    }
    if( controlWidget() ) {
        eventView()->removeChild( controlWidget() );
        delete controlWidget();
    }
    if( hasKeepWidget() && keepWidget() ) {
        eventView()->removeChild( keepWidget() );
        delete keepWidget();
    }
}

void EventViewItem::setMarginLeft(int margin) {
    if( m_hasControlDelete && controlWidget() ) {
        QSize sC = controlWidget()->sizeHint();
        m_marginLeft = QMAX(margin,sC.width());
        if ( m_hasControlKeep && keepWidget() ) {
            QSize sK = keepWidget()->sizeHint();
            m_marginLeft = QMAX(margin,sC.width() + sK.width());
        }

    } else {
        m_marginLeft = margin;
    }
}

int EventViewItem::marginLeft() const {
    return m_marginLeft;
}

void EventViewItem::setEventItem(const EventItem* item)
{
    m_eventItem = item;

    if( item ) {
        m_hasAction = item->data.action().actionType() == UrlAction;

        if( m_hasAction ) {
            if( actionWidget() )
                eventView()->removeChild( actionWidget() );
            EventActionButton *actionButton =
                    new EventActionButton( item->id, item->data.action(), eventView()->viewport(), "action_button" );
            if( m_smallActionButton )
                actionButton->setPixmap( KGlobal::iconLoader()->loadIcon("launch",KIcon::Panel,0,false) );
            else
                actionButton->setText( item->data.action().actionText() );
            QToolTip::add(actionButton, item->data.action().actionText());
            eventView()->addChild( actionButton );
            setActionWidget(actionButton);
            actionButton->resize( actionButton->sizeHint() );
//             connect( actionButton, SIGNAL(clicked()), this, SLOT(slotActionClicked()) );
        } else {
            if( actionWidget() )
                eventView()->removeChild( actionWidget() );
            delete actionWidget();
            setActionWidget( 0 );
        }


        // load icons into the mimesource...
        if( !item->data.icon().isEmpty() )
            eventView()->loadMimeSource( item->data.icon() );
        // also extract and load img sources from the text(). ...and please: forgive my low-skilled regexp voodoo..... :o)
        QRegExp imgsrc( "(?:<|&lt;)\\s?img(?:[a-z\\s=]|\".*\"|'.*')*src\\s*=\\s*[\"'](http://[^\"']*\\.(?:gif|jpg|png|bmp))(?:[a-z\\s=]|\".*\"|'.*')*(?:>|&gt;)" );
        int pos = imgsrc.search( item->data.text() );
        if ( pos > -1 ) {
            QStringList urls = imgsrc.capturedTexts();
            // start with ++urls.begin() because we don't need the first match which only
            // contains the regexp'd string ("<img ...>" in this case)...
            for(QStringList::Iterator it = ++urls.begin(); it != urls.end(); ++it) {
                eventView()->loadMimeSource( (*it) );
            }
        }

        QString eventDisplayString;
        eventDisplayString += QString("<p><font size='-1'>%1</font>")
                .arg(KGlobal::locale()->formatDateTime(QDateTime(item->data.date(),item->data.time())));
        eventDisplayString += "<table border='0'>";
        if( !grouped() ) {
            eventDisplayString += QString("<tr><td width='16'><img source='%1' width='16' height='16'></td><td><b>%2</b></td></tr>")
                    .arg(item->data.icon()).arg(item->data.title());
        }
        if( !item->data.text().isEmpty() ) {
            eventDisplayString += QString("<tr><td width='16'></td><td>%1</td></tr>")
                    .arg(item->data.text());
        }
        eventDisplayString += "</table></p>";
        delete m_rt;
        m_rt = new QSimpleRichText(eventDisplayString, eventView()->viewport()->font(),
                QString::null, 0, eventView()->mimeSourceFactory() );
    } else {
        delete actionWidget();
        setActionWidget( 0 );
        delete m_rt;
        m_rt = 0;
    }
}

const EventItem* EventViewItem::eventItem() const
{
    return m_eventItem;
}

bool EventViewItem::hasActionWidget() const
{
    return m_hasAction;
}

bool EventViewItem::hasControlWidget() const
{
    return m_hasControlDelete;
}

bool EventViewItem::hasKeepWidget() const
{
    return m_hasControlKeep;
}

QRect EventViewItem::actionWidgetGeometry() const
{
    if( m_hasAction && actionWidget() ) {
        QSize sA = actionWidget()->sizeHint();
        if (m_smallActionButton)
            return QRect( 5, height()-sA.height()-5, sA.width(), sA.height() );
        else
            return QRect( 50, height()-sA.height()-5, sA.width(), sA.height() );
    } else {
        return QRect();
    }
}

QRect EventViewItem::controlWidgetGeometry() const
{
    if( m_hasControlDelete && controlWidget() ) {
        QSize sC = controlWidget()->sizeHint();
        return QRect( 5, 5, sC.width(), sC.height() );
    } else {
        return QRect();
    }
}

QRect EventViewItem::keepWidgetGeometry() const
{
    if( m_hasControlKeep && keepWidget() ) {
        QSize sK = keepWidget()->sizeHint();
        return QRect( 5, 5, sK.width(), sK.height() );
    } else {
        return QRect();
    }
}

void EventViewItem::slotControlClicked()
{
    if( m_hasControlDelete )
        emit(deleteItem(this) );
}

void EventViewItem::slotKeepClicked()
{
    if( m_hasControlKeep )
        emit(keepItem(this) );
}

int EventViewItem::heightForWidth(int width) const
{
    if( m_rt ) {
        m_rt->setWidth(width-marginLeft());
         // for some reason the richtext painter has to much space at the bottom when using a <table> in
        // the richtext source. so shorten the thing by 20 pixel...
        int textHeight = m_rt->height() - 20;
        if( m_hasAction && !m_smallActionButton && actionWidget() ) {
            return 5 + textHeight + 5 + actionWidget()->height() + 5 + 1; // 5 pixel spacing...
        } else {
            return 5 + textHeight + 5;
        }
    } else {
        return 0;
    }
}

void EventViewItem::drawContents( QPainter* p, const QPoint& pos, const QRect& cr ) const
{
    QColorGroup g =  eventView()->viewport()->colorGroup();
    if( selected() )
        g.setColor( QColorGroup::Text, g.highlightedText() );
    QBrush bg = selected() ? g.highlight() : g.base();
    QColor fg = selected() ? g.highlightedText() : g.foreground();

    int x = pos.x();
    int y = pos.y();
    int w = width();
    int h = height();

    p->fillRect( x,y,w,h , bg );
    if( m_rt ) {
        int rtx = x+marginLeft();
        int rty = y+5; // 5 pixels top spacing...
        m_rt->draw( p, rtx, rty, cr , g );
    }

    p->setPen( QPen(fg, 0, QPen::DotLine) );
    p->drawLine(x, y+h-1, x+w, y+h-1);
}

EventViewHeaderItem::EventViewHeaderItem(EventView *ev) : EventViewItemBase(ev),
  m_displayChildNumber(false)
{
    // create button...
    m_button = new PlusMinusButton(eventView()->viewport(), "expand_button");
    eventView()->addChild( m_button );
    setControlWidget(m_button);
//     m_button->setOn( !showChildItems() );
    m_button->resize( m_button->sizeHint() );
    connect( m_button, SIGNAL(toggled(bool)), this, SLOT(slotControlToggled(bool)) );
}

EventViewHeaderItem::~EventViewHeaderItem()
{
    if( controlWidget() ) {
        eventView()->removeChild( controlWidget() );
        delete controlWidget();
    }
}

QRect EventViewHeaderItem::controlWidgetGeometry() const
{
    int h = height();
    return QRect( 10, (h/2)-(m_button->height()/2), m_button->width(), m_button->height() );
}

/*
QRect EventViewHeaderItem::keepWidgetGeometry() const
{
    int h = height();
    return QRect( 10, (h/2)-(m_button->height()/2), m_button->width(), m_button->height() );
}
*/

void EventViewHeaderItem::drawContents( QPainter* p, const QPoint& pos, const QRect& ) const
{
    int children = childItems().count();

    expandButton()->setEnabled( children >= 1 );
    expandButton()->setOn( !showChildItems() );

    QColorGroup g =  eventView()->viewport()->colorGroup();
    QBrush bg = g.mid();
    QColor fg = (children>=1)?g.text():g.dark();

    int x = pos.x();
    int y = pos.y();
    int w = width();
    int h = height();

    p->fillRect( x,y,w,h ,bg );
    QFont font( eventView()->viewport()->font() );
    font.setBold(true);
    p->setPen(fg);
    p->setFont( font );
    QString displayString = text();
    if( displayChildNumber() )
        displayString += QString(" (%1)").arg(children);
    p->drawText( x+30,y+5,w-30,h-5, Qt::AlignLeft|Qt::SingleLine, displayString );

    p->setPen( QPen(fg, 0, QPen::SolidLine) );
    p->drawLine(x, y+h-1, x+w, y+h-1);
}

int EventViewHeaderItem::heightForWidth(int) const
{
    QFont font( eventView()->viewport()->font() );
    font.setBold(true);
    QFontMetrics fm(font);
    return 3 + fm.lineSpacing() + 7; // a few extra pixels for the frame etc.
}

void EventViewHeaderItem::slotControlToggled(bool on)
{
    setShowChildItems( !on );
}

EventViewGroupItem::EventViewGroupItem(EventView* ev) : EventViewHeaderItem(ev),
  m_hasIcon(false)
{
}

EventViewGroupItem::~EventViewGroupItem()
{
}

void EventViewGroupItem::setIcon(const QString& icon)
{
    m_icon = icon;
    m_hasIcon = true;

    // load icon into the mimesource...
    if( !icon.isEmpty() )
        eventView()->loadMimeSource( icon );
}

QRect EventViewGroupItem::controlWidgetGeometry() const
{
    int h = height();
    return QRect( 30, (h/2)-(controlWidget()->height()/2), controlWidget()->width(), controlWidget()->height() );
}

void EventViewGroupItem::drawContents( QPainter* p, const QPoint& pos, const QRect& ) const
{
    const int leftMargin = 50;

    int children = childItems().count();

    expandButton()->setEnabled( children >= 1 );
    expandButton()->setOn( !showChildItems() );

    QColorGroup g =  eventView()->viewport()->colorGroup();
    QBrush bg = g.mid();
    QColor fg = (children>=1)?g.text():g.dark();

    int x = pos.x();
    int y = pos.y();
    int w = width();
    int h = height();

    p->fillRect( x,y,w,h ,bg );
    QFont font( eventView()->viewport()->font() );
    font.setBold(true);
    p->setPen(fg);
    p->setFont( font );
    QRect textRect;
    if( m_hasIcon ) {
        QPixmap pm( QPixmap::fromMimeSource(m_icon) );
        int margin = leftMargin+pm.width()+5;
        textRect = QRect( x+margin,y+3,w-margin,h-3 );
        // also draw the icon while we are at it...
        p->drawPixmap( x+leftMargin, y+(height()/2)-(pm.height()/2), pm );
    } else {
        textRect = QRect( x+leftMargin,y+3,w-leftMargin,h-3 );
    }
    QString displayString = text();
    if( displayChildNumber() )
        displayString += QString(" (%1)").arg(children);
    p->drawText( textRect , Qt::AlignLeft|Qt::SingleLine, displayString );

    p->setPen( QPen(fg, 0, QPen::DotLine) );
    p->drawLine(x, y+h-1, x+w, y+h-1);
}

int EventViewGroupItem::heightForWidth(int) const
{
    QFont font( eventView()->viewport()->font() );
    font.setBold(true);
    QFontMetrics fm(font);
    int height = 2 + fm.lineSpacing() + 4; // a few extra pixels for the frame etc.
    int pixmapHeight = 0;
    if( m_hasIcon ) {
        QPixmap pm( QPixmap::fromMimeSource(m_icon) );
        pixmapHeight = pm.height() + 2; // 2 px space...
    }
    return QMAX(height,pixmapHeight);
}

};

#include "mainwidget.moc"
