/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h>
#include <qlineedit.h>
#include <qpainter.h>
#include <qptrlist.h> 
#include <qspinbox.h>
#include <qstringlist.h>
#include <qtextedit.h>
#include <qvaluevector.h>
#include <qwhatsthis.h>

#include <kdebug.h>

#include <kcolorbutton.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klistview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kurl.h>
#include <kurlrequester.h>
#include <kpushbutton.h>

#include "config.h"
#include "configdialog.h"
#include "sourcedlg.h"

#define FILTEROUT_DEFAULT "run-crons,lastrun/cron.hourly," + i18n("[your-ignore-list]")

LogConfig::LogConfig(QWidget* parent)
    : QObject(parent), m_config(0), m_dialog(0)
{
    // create the configuration object
    m_config = new KConfig("eventwatcherlogpluginrc");
    KGlobal::locale()->insertCatalogue("eventwatcherlogplugin_config");

    // create and show the configuration dialog
    m_dialog = new ConfigDialog(parent);
    // set the command buttons to a suitable initial state...
    m_dialog->addButton->setEnabled(true);
    m_dialog->modifyButton->setEnabled(false);
    m_dialog->removeButton->setEnabled(false);
    m_dialog->show();

    // setup the list view...
    m_dialog->sourceListView->setColumnWidthMode( 0, QListView::Maximum );
    m_dialog->sourceListView->setColumnWidthMode( 1, QListView::Manual );
    m_dialog->sourceListView->setColumnWidth( 1, 50 );
    m_dialog->sourceListView->setColumnAlignment( 1, AlignCenter );

    // load the configuration
    load();
    updateSourceListView();

    // setup the connections
    connect( m_dialog->sourceListView, SIGNAL(selectionChanged()), this, SLOT(updateButtons()) );
    connect( m_dialog->addButton, SIGNAL(clicked()), this, SLOT(slotAdd()) );
    connect( m_dialog->modifyButton, SIGNAL(clicked()), this, SLOT(slotModify()) );
    connect( m_dialog->removeButton, SIGNAL(clicked()), this, SLOT(slotRemove()) );
    connect(m_dialog->intervalSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));
    connect(m_dialog->ignoreLineEdit, SIGNAL(textChanged(const QString&)), this, SIGNAL(changed()));
}

LogConfig::~LogConfig()
{
    delete m_dialog;
    delete m_config;
}

void LogConfig::slotAdd()
{
    SourceDialog add(m_dialog, "add_dialog", true); // modal, child of the dialog widget...

    // set deault color...
    add.m_color->setColor( Qt::black );

    bool accepted = false;
    while(add.exec()==QDialog::Accepted)
    {
        if( KURL(add.m_source->url()).isLocalFile() ) {
            accepted = true;
            break;
        } else {
            KMessageBox::sorry( m_dialog, i18n("Please make sure the log file you specified is a local file.") );
        }
    }
    if( accepted ) {
        SourceInfo info;
        info.enabled = true;
        info.url = add.m_source->url();
        info.color = add.m_color->color();
        m_sourceList.append(info);
        updateSourceListView(); // refresh the list...
        emit( changed() ); // enable apply button etc.

        // warn if the file isn't accessible
        checkFile( info.url );
    }
}

void LogConfig::slotModify()
{
    SourceDialog edit(m_dialog, "add_dialog", true); // modal, child of the dialog widget...

    // get the currently selected item...
    SourceListItem *listItem = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(!listItem)
        return;

    SourceInfo &info = listItem->info();

    // update the dialog...
    edit.m_source->setURL( info.url );
    edit.m_color->setColor( info.color );

    bool accepted = false;
    while(edit.exec()==QDialog::Accepted)
    {
        if( KURL(edit.m_source->url()).isLocalFile() ) {
            accepted = true;
            break;
        } else {
            KMessageBox::sorry( m_dialog, i18n("Please make sure the log file you specified is a local file.") );
        }
    }
    if( accepted ) {
        info.url = edit.m_source->url();
        info.color = edit.m_color->color();
        updateSourceListView(); // refresh the list...
        emit( changed() ); // enable apply button etc.

        // warn if the file isn't accessible
        checkFile( info.url );
    }
}

void LogConfig::slotRemove()
{
    SourceListItem *listItem = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(!listItem)
        return;

    m_sourceList.remove( listItem->info() );

    updateSourceListView(); // refresh the list...
    emit( changed() ); // enable apply button etc.
}

void LogConfig::load()
{
    m_config->setGroup("General");
    m_dialog->intervalSpinBox->setValue( m_config->readNumEntry("Interval", 10) );
    QStringList filters;
    if( m_config->hasKey("FilterOut") )
        filters = m_config->readListEntry("FilterOut" );
    else
        filters = QStringList::split(',',FILTEROUT_DEFAULT);
    m_dialog->ignoreLineEdit->setText( filters.join(",") );
    QStringList sources = m_config->readListEntry("SourceIndex");
    m_config->setGroup("Sources");
    m_sourceList.clear();
    for(QStringList::Iterator it = sources.begin(); it != sources.end(); ++it) {
        SourceInfo info;
        info.enabled = m_config->readBoolEntry( (*it) + "_isEnabled", false );
        QColor defaultColor(Qt::black);
        info.color = m_config->readColorEntry( (*it) + "_Color", &defaultColor );
        info.url = (*it);
        m_sourceList.append(info);
    }
}

void LogConfig::save()
{
    m_config->setGroup("General");
    m_config->writeEntry( "Interval", m_dialog->intervalSpinBox->value() );
    QStringList filters = QStringList::split(',', m_dialog->ignoreLineEdit->text());
    m_config->writeEntry("FilterOut", filters);
    m_config->deleteGroup( "Sources" );
    m_config->setGroup("Sources");
    QStringList index;
    // write all sources...
    for(SourceList::Iterator it = m_sourceList.begin(); it != m_sourceList.end(); ++it) {
        index.append( (*it).url );
        m_config->writeEntry( (*it).url + "_isEnabled", (*it).enabled );
        m_config->writeEntry( (*it).url + "_Color", (*it).color );
    }
    m_config->setGroup("General");
    m_config->writeEntry( "SourceIndex", index ); // write source index...

    m_config->sync();
}

void LogConfig::defaults()
{
    m_dialog->intervalSpinBox->setValue(10);

    m_dialog->ignoreLineEdit->setText( FILTEROUT_DEFAULT );

    m_sourceList.clear();
    // default file sources:
    SourceInfo info;
    info.enabled = false;
    info.color = QColor( 0, 0, 50 );
    info.url = "/var/log/messages";
    m_sourceList.append(info);

    updateSourceListView();
}

void LogConfig::updateButtons()
{
    SourceListItem *i = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(i) {
        m_dialog->modifyButton->setEnabled(true);
        m_dialog->removeButton->setEnabled(true);
    } else {
        m_dialog->modifyButton->setEnabled(false);
        m_dialog->removeButton->setEnabled(false);
    }
    m_dialog->addButton->setEnabled(true);
}

void LogConfig::listItemStateChanged(SourceInfo& i)
{
    if( i.enabled ) {
        checkFile( i.url );
    }
}

bool LogConfig::checkFile( QString url )
{
    QFile file( url );
    if( !file.open( IO_ReadOnly ) ) {
        KMessageBox::detailedSorry( m_dialog, i18n("Could not open log file \"%1\" for reading. Please make sure the log file you specified exists and you have the rights to access it.").arg(url), file.errorString() );
        return false;
    }

    return true;
}

void LogConfig::updateSourceListView()
{
    m_dialog->sourceListView->clear(); // remove old entries

    // fill sources...
    for(SourceList::Iterator it = m_sourceList.begin(); it != m_sourceList.end(); ++it) {
        SourceListItem *item = new SourceListItem( *it, m_dialog->sourceListView );
        connect( item, SIGNAL(stateChanged(SourceInfo&)), this, SIGNAL(changed()) );
        connect( item, SIGNAL(stateChanged(SourceInfo&)), this, SLOT(listItemStateChanged(SourceInfo&)) );
    }
}

SourceListItem::SourceListItem(SourceInfo& info, QListView *parent)
 : QCheckListItem(parent, info.url, CheckBox),
   m_info(info)
{
    setOn(m_info.enabled);

    updateColor();
}

SourceListItem::SourceListItem(SourceInfo& info, QListViewItem *parent)
 : QCheckListItem(parent, info.url, CheckBox),
   m_info(info)
{
    setOn(m_info.enabled);

    updateColor();
}

void SourceListItem::updateColor()
{
    QPixmap pix(16,16);
    QPainter p( &pix );
    p.fillRect( pix.rect(), m_info.color );
    p.end();
    setPixmap(1, pix);
}

void SourceListItem::stateChange(bool b)
{
    m_info.enabled = b;
    emit(stateChanged(m_info));
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    QObject* allocate_config(QWidget* parent) {
        return (new LogConfig(parent));
    }
}

#include "config.moc"
