/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h>
#include <qmap.h>
#include <qtimer.h>

#include <kdebug.h>

#include <kaction.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include "../../lib/eventdata.h"

#include "logplugin.moc"
#include "logplugin.h"

#define FILTEROUT_DEFAULT "run-crons,lastrun/cron.hourly"

extern "C"
{
    void* init_eventwatcherlogplugin()
    {
        KGlobal::locale()->insertCatalogue("eventwatcherlog");
        return new EventWatcherLogPlugin::Factory;
    }
}

namespace EventWatcherLogPlugin {

Factory::Factory()
{
    s_instance = new KInstance( "eventwatcher" );
}

Factory::~Factory()
{
    delete s_instance;
}

QObject* Factory::createObject( QObject* parent, const char* name, const char*, const QStringList & )
{
    return new LogPlugin( parent, name );
}

KInstance* Factory::s_instance = 0L;

LogPlugin::LogPlugin( QObject* parent, const char* name )
    : EventWatcher::Plugin ( parent, name )
{
    m_sources.setAutoDelete(true);

    readConfig();

    m_timer = new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(update()) );
    m_timer->start( m_interval * 1000, false ); // 1 min = 1000 msec ...

    update();
}

LogPlugin::~LogPlugin()
{
}

void LogPlugin::settingsChanged()
{
    readConfig();

    update();

    m_timer->changeInterval( m_interval * 1000 ); // 1 sec = 1000 msec ...
}

void LogPlugin::onlineSettingChanged()
{
    // can only operate on local files, so we don't
    // need to care about the online setting...
}

void LogPlugin::performUpdateCheck()
{
    update();
}

void LogPlugin::update()
{
    // pause the timer during the check...
    m_timer->stop();

    QString eventText;
    int logNumber = 0;

    for( SourceInfoList::Iterator it = m_sources.begin(); it != m_sources.end(); ++it ) {
        SourceInfo *i = (*it);
        if( ! i->file.isOpen() ) {
            if( i->file.open( IO_ReadOnly ) ) {
                // Go to the end
                i->file.at( i->file.size() );
            } else {
                kdWarning() << i18n("Could not open log file \"%1\" for reading: %2").arg( i->url ).arg( i->file.errorString() ) << endl;
            }
        } else {
            // file is open and the offset should be at the position of our
            // last check. now read new lines if there are any.
            QString logText,
                    line;
            while( (i->file.readLine(line, 2048)) > 0 ) {
                bool filter = false;
                for(QStringList::Iterator it = m_filters.begin(); it != m_filters.end(); ++it) {
                    if( line.contains( *it ) ) {
                        filter = true;
                        break;
                    }
                }
                if( !filter )
                    logText += line;
            }

            if( !logText.isEmpty() ) {
                ++logNumber;
                /*// add line break between log files...
                if( logNumber >= 2 )
                eventText += "<br>"; */
                // also add line break between log lines
		//eventText += "<font color=\"" + i->color.name() + "\">" + logText.replace("\n", "<br>") + "</font>";

		// (eldad 25 Sep 2005) Help grouping by specifying log filename and sending event for each logfile.
		QString logTextHTML = "<font color=\"" + i->color.name() + "\">" + logText.replace("\n", "<br>") + "</font>";
		QString title = i18n("Log file update") + " (" + i->url + ")";
		EventWatcher::EventData data;
        	data.setData( title, title, logTextHTML , "logplugin" );
        	createEvent( data );
            }
        }
    }

    m_timer->start( m_interval * 1000, false );
}

void LogPlugin::readConfig()
{
    KConfig config("eventwatcherlogpluginrc");
    config.setGroup("General");
    m_interval = config.readNumEntry("Interval", 10);
    if( config.hasKey("FilterOut") )
        m_filters = config.readListEntry("FilterOut" );
    else
        m_filters = QStringList::split(',',FILTEROUT_DEFAULT);
    QStringList sources = config.readListEntry("SourceIndex");
    config.setGroup("Sources");
    m_sources.clear();
    for(QStringList::Iterator it = sources.begin(); it != sources.end(); ++it) {
        if( config.readBoolEntry( (*it) + "_isEnabled", false ) ) {
            SourceInfo *info = new SourceInfo;
            QColor defaultColor(Qt::black);
            info->color = config.readColorEntry( (*it) + "_Color", &defaultColor );
            info->url = (*it);
            info->file.setName( *it );
            m_sources.append(info);
        }
    }
}

};
