/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qlineedit.h>
#include <qptrlist.h> 
#include <qspinbox.h>
#include <qstringlist.h>
#include <qtextedit.h>
#include <qvaluevector.h>
#include <qwhatsthis.h>

#include <kdebug.h>

#include <kconfig.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klistview.h>
#include <klocale.h>
#include <kurlrequester.h>
#include <kpushbutton.h>

#include "config.h"
#include "configdialog.h"
#include "sourcedlg.h"
#include "newsfeeds.h"

RssConfig::RssConfig(QWidget* parent)
    : QObject(parent), m_config(0), m_dialog(0)
{
    // create the configuration object
    m_config = new KConfig("eventwatcherrsspluginrc");
    KGlobal::locale()->insertCatalogue("eventwatcherrssplugin_config");

    // create and show the configuration dialog
    m_dialog = new ConfigDialog(parent);
    // set the command buttons to a suitable initial state...
    m_dialog->addButton->setEnabled(true);
    m_dialog->modifyButton->setEnabled(false);
    m_dialog->removeButton->setEnabled(false);
    m_dialog->show();

    m_dialog->sourceListView->setRootIsDecorated( true );

    // load the configuration
    load();
    updateSourceListView();

    // setup the connections
    connect( m_dialog->sourceListView, SIGNAL(selectionChanged()), this, SLOT(updateButtons()) );
    connect( m_dialog->addButton, SIGNAL(clicked()), this, SLOT(slotAdd()) );
    connect( m_dialog->modifyButton, SIGNAL(clicked()), this, SLOT(slotModify()) );
    connect( m_dialog->removeButton, SIGNAL(clicked()), this, SLOT(slotRemove()) );
    connect(m_dialog->intervalSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));
}

RssConfig::~RssConfig()
{
    delete m_dialog;
    delete m_config;
}

void RssConfig::slotAdd()
{
    SourceDialog add(m_dialog, "add_dialog", true); // modal, child of the dialog widget...

    if(add.exec()==QDialog::Accepted)
    {
        SourceInfo info;
        info.enabled = true;
        info.name = add.m_name->text();
        info.url = add.m_source->url();
        m_customSourceList.append(info);
        updateSourceListView(); // refresh the list...
        emit( changed() ); // enable apply button etc.
    }
}

void RssConfig::slotModify()
{
    SourceDialog edit(m_dialog, "add_dialog", true); // modal, child of the dialog widget...

    // get the currently selected item...
    SourceListItem *listItem = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(!listItem)
        return;

    SourceInfo &info = listItem->info();

    // update the dialog...
    edit.m_name->setText( info.name );
    edit.m_source->setURL( info.url.url() );

    if(edit.exec()==QDialog::Accepted)
    {
        if( info.custom ) {
            info.name = edit.m_name->text();
            info.url = edit.m_source->url();
        } else {
            // don't touch default sources, so move modified default ones into the custom list...
            SourceInfo newInfo;
            newInfo.enabled = info.enabled;
            newInfo.name = edit.m_name->text();
            newInfo.url = edit.m_source->url();
            m_defaultSourceList.remove( info.name ); // remove from default list...
            m_customSourceList.append(newInfo); // add to custom list...
        }
        updateSourceListView(); // refresh the list...
        emit( changed() ); // enable apply button etc.
    }
}

void RssConfig::slotRemove()
{
    SourceListItem *listItem = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(!listItem)
        return;
    else if( !listItem->info().custom )
        return;

    m_customSourceList.remove( listItem->info() );

    updateSourceListView(); // refresh the list...
    emit( changed() ); // enable apply button etc.
}

void RssConfig::load()
{
    m_config->setGroup("General");
    m_dialog->intervalSpinBox->setValue( m_config->readNumEntry("Interval", 5) );
    QStringList sources = m_config->readListEntry("SourceIndex");
    m_config->setGroup("Sources");
    m_customSourceList.clear();
    m_defaultSourceList.clear();
    for(QStringList::Iterator it = sources.begin(); it != sources.end(); ++it) {
        bool custom = m_config->readBoolEntry( (*it) + "_isCustom", false );
        SourceInfo info;
        info.custom = custom;
        info.enabled = m_config->readBoolEntry( (*it) + "_isEnabled", false );
        info.url = KURL( m_config->readEntry( (*it) + "_Url" ) );
        info.name = (*it);
        if( custom )
            m_customSourceList.append(info);
        else
            m_defaultSourceList[*it] = info;
    }
}

void RssConfig::save()
{
    m_config->setGroup("General");
    m_config->writeEntry( "Interval", m_dialog->intervalSpinBox->value() );
    m_config->deleteGroup( "Sources" );
    m_config->setGroup("Sources");
    QStringList index;
    // write all default sources
    for(DefaultSourceList::Iterator it = m_defaultSourceList.begin(); it != m_defaultSourceList.end(); ++it) {
        // only enabled ones...
        if( (*it).enabled ) {
            index.append( (*it).name );
            m_config->writeEntry( (*it).name + "_isEnabled", (*it).enabled );
            m_config->writeEntry( (*it).name + "_isCustom", false );
            m_config->writeEntry( (*it).name + "_Url", (*it).url.url() );
        }
    }
    // write all custom sources
    for(CustomSourceList::Iterator it = m_customSourceList.begin(); it != m_customSourceList.end(); ++it) {
        index.append( (*it).name );
        m_config->writeEntry( (*it).name + "_isEnabled", (*it).enabled );
        m_config->writeEntry( (*it).name + "_isCustom", true );
        m_config->writeEntry( (*it).name + "_Url", (*it).url.url() );
    }
    m_config->setGroup("General");
    m_config->writeEntry( "SourceIndex", index ); // write source index...

    m_config->sync();
}

void RssConfig::defaults()
{
    m_dialog->intervalSpinBox->setValue(5);
    m_defaultSourceList.clear();
    m_customSourceList.clear();
    updateSourceListView();
}

void RssConfig::updateButtons()
{
    SourceListItem *i = dynamic_cast<SourceListItem*>(m_dialog->sourceListView->selectedItem());
    if(i) {
        m_dialog->modifyButton->setEnabled(true);
        if( i->info().custom )
            m_dialog->removeButton->setEnabled(true);
        else
            m_dialog->removeButton->setEnabled(false);
    } else {
        m_dialog->modifyButton->setEnabled(false);
        m_dialog->removeButton->setEnabled(false);
    }
    m_dialog->addButton->setEnabled(true);
}

void RssConfig::updateSourceListView()
{
    m_dialog->sourceListView->clear(); // remove old entries

    QValueVector<QListViewItem*> parents;
    QValueVector<QListViewItem*>::Iterator it;
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Arts" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Business" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Computers/Misc" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Computers/KDE" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Misc" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Magazines" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Recreation" ) ) );
    parents.append( new QListViewItem( m_dialog->sourceListView, i18n( "Society" ) ) );
    for ( it = parents.begin(); it != parents.end(); ++it ) {
        (*it)->setSelectable( false );
    }

    // fill default news sources
    for ( int i = 0; i < DEFAULT_NEWSSOURCES; ++i ) {
        NewsSourceData data = NewsSourceDefault[ i ];
        SourceListItem *item = 0;
        if( m_defaultSourceList.contains(data.name()) && !m_defaultSourceList[data.name()].custom ) {
            SourceInfo &i = m_defaultSourceList[data.name()];
            item = new SourceListItem(i, parents[ data.category() ] );
            // open the tree in case it contains enabled children..
            if( i.enabled )
                parents[ data.category() ]->setOpen( true );
        } else {
            SourceInfo info;
            info.custom = false;
            info.enabled = false;
            info.name = data.name();
            info.url = data.url();
            m_defaultSourceList[data.name()] = info;
            item = new SourceListItem( m_defaultSourceList[data.name()], parents[ data.category() ] );
        }
        connect( item, SIGNAL(stateChanged(SourceInfo&)), this, SIGNAL(changed()) );
    }

    // fill custom news sources...
    QListViewItem *custom = new QListViewItem( m_dialog->sourceListView, i18n( "Custom" ) );
    custom->setSelectable( false );
    custom->setOpen( true );
    custom->setPixmap( 0, KGlobal::iconLoader()->loadIcon("edit",KIcon::Panel,KIcon::SizeSmall) );
    // hide the custom tree in case there aren't any children...
    if( m_customSourceList.count() == 0 )
        custom->setVisible( false );
    for(CustomSourceList::Iterator it = m_customSourceList.begin(); it != m_customSourceList.end(); ++it) {
        SourceListItem *item = new SourceListItem( *it, custom );
        connect( item, SIGNAL(stateChanged(SourceInfo&)), this, SIGNAL(changed()) );
    }
}

SourceListItem::SourceListItem(SourceInfo& info, QListView *parent)
 : QCheckListItem(parent, info.name, CheckBox),
   m_info(info)
{
//     setText ( 1, info->url.url() );
    setOn(m_info.enabled);
}

SourceListItem::SourceListItem(SourceInfo& info, QListViewItem *parent)
 : QCheckListItem(parent, info.name, CheckBox),
   m_info(info)
{
//     setText ( 1, info->url.url() );
    setOn(m_info.enabled);
}

void SourceListItem::stateChange(bool b)
{
    m_info.enabled = b;
    emit(stateChanged(m_info));
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    QObject* allocate_config(QWidget* parent) {
        return (new RssConfig(parent));
    }
}

#include "config.moc"
