/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h>
#include <qmap.h>
#include <qtimer.h>

#include <kdebug.h>

#include <kaction.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include <dcopclient.h>

// #include <rss/article.h>

#include "../../lib/eventdata.h"

#include "rssplugin.moc"
#include "rssplugin.h"

extern "C"
{
    void* init_eventwatcherrssplugin()
    {
        KGlobal::locale()->insertCatalogue("eventwatcherrss");
        return new EventWatcherRssPlugin::Factory;
    }
}

namespace EventWatcherRssPlugin {

Factory::Factory()
{
    s_instance = new KInstance( "eventwatcher" );
}

Factory::~Factory()
{
    delete s_instance;
}

QObject* Factory::createObject( QObject* parent, const char* name, const char*, const QStringList & )
{
  return new RssPlugin( parent, name );
}

KInstance* Factory::s_instance = 0L;

RssPlugin::RssPlugin( QObject* parent, const char* name )
    : EventWatcher::Plugin ( parent, name )
{
    readConfig();

    // try to start the dcop service if necessary...
    m_serviceAvailable = true;
    if ( !kapp->dcopClient()->isApplicationRegistered( "rssservice" ) ) {
        if ( KApplication::startServiceByDesktopName("rssservice") != 0 ) {
            kdWarning() << "Failed to initialize the rss dcop service. The rss plugin will be unable to work." << endl;
            m_serviceAvailable = false;
        }
    }

    // refresh timer is used to refresh the rss documents.
    connect( &m_refreshTimer, SIGNAL(timeout()), this, SLOT(refreshDocuments()) );

    // react on removed documents...
    connectDCOPSignal( 0, 0, "removed(QString)", "documentRemoved(QString)", false );

    m_initialized = false;
    initDocuments();

//     update();
//     m_timer = new QTimer(this);
//     connect(m_timer, SIGNAL(timeout()), this, SLOT(update()) );
//     onlineSettingChanged(); // will start the timer if necessary...
}

RssPlugin::~RssPlugin()
{
    // clean up and remove my documents from the service list.
    DCOPRef dcopCall( "rssservice", "RSSService" );
    for( QStringList::const_iterator it = m_sources.begin(); it != m_sources.end(); ++it ) {
        dcopCall.send("remove(QString)", *it);
    }
}

void RssPlugin::settingsChanged()
{
    readConfig();
    initDocuments();
}

void RssPlugin::onlineSettingChanged()
{
    initDocuments();
}

void RssPlugin::performUpdateCheck()
{
    refreshDocuments();
}

void RssPlugin::initDocuments()
{
    m_initialized = false;

    m_feeds.clear(); // clear the feed list...

    // stop here if the service isn't available.
    if( !m_serviceAvailable ) return;

    // add all (m_)sources to the service
    DCOPRef dcopCall( "rssservice", "RSSService" );
    for(QStringList::const_iterator it = m_sources.begin(); it != m_sources.end(); ++it) {
        DCOPRef feedRef = dcopCall.call("add(QString)", *it);
        m_feeds.append(feedRef);
        connectDCOPSignal( "rssservice", feedRef.obj(), "documentUpdated(DCOPRef)",
                            "documentUpdated(DCOPRef)", false );
    }

    m_initialized = true;

    refreshDocuments();
}

void RssPlugin::refreshDocuments()
{
    m_refreshTimer.stop();

    if( !m_initialized )
        return;

    for( FeedList::iterator it = m_feeds.begin(); it != m_feeds.end(); ++it ) {
        (*it).send( "refresh()" );
    }

    m_refreshTimer.start( m_interval*60*1000 );
}

void RssPlugin::documentUpdated( DCOPRef ref )
{
    if( ref.isNull() )
        return;

    QString docTitle;
    ref.call( "title()" ).get( docTitle );

    HeadlineList oldHeadlines = getOldHeadlines(docTitle);
    HeadlineList newHeadlines;

    int articleCount;
    ref.call( "count()" ).get( articleCount );
    for( int i = 0; i < articleCount; ++i ) {
        DCOPRef article = ref.call( "article(int)", i );
        QString title;
        article.call( "title()" ).get( title );
        if( ! oldHeadlines.contains(title) ) {
            QString description, link;
            article.call( "description()" ).get( description );
            article.call( "link()" ).get( link );
            QString msg = "<i>"+title+"</i>";
            if( !description.isEmpty() )
                msg += "<br>"+description;
            EventWatcher::EventAction action( EventWatcher::UrlAction, link, i18n("Visit") );
            EventWatcher::EventData data(i18n("RSS feed update"),
                    i18n("RSS feed \"%1\" has been updated").arg(docTitle), msg, "rssplugin", action );
            createEvent(data);
        }
        newHeadlines[title] = true;
    }
    setOldHeadlines(newHeadlines, docTitle);
}

void RssPlugin::documentRemoved( QString )
{
    // in case a document we are currently using should have been removed, this
    // will add it again.
    kdDebug() << "a document has been removed." << endl;
    initDocuments();
}

void RssPlugin::readConfig()
{
    KConfig config("eventwatcherrsspluginrc");

    config.setGroup("General");
    m_interval = config.readNumEntry("Interval", 60); //Be nice by default
    QStringList sources = config.readListEntry("SourceIndex");
    config.setGroup("Sources");
    m_sources.clear();
    for(QStringList::Iterator it = sources.begin(); it != sources.end(); ++it) {
        if( config.readBoolEntry( (*it) + "_isEnabled", false ) ) {
            m_sources.append( config.readEntry((*it)+"_Url") );
        }
    }
}

HeadlineList RssPlugin::getOldHeadlines(const QString& newsFeed)
{
    QString fileName = locateLocal("appdata", "rss/"+newsFeed+".read");
    QFile file(fileName);
    file.open(IO_ReadOnly);

    QTextStream stream(&file);
    stream.setEncoding(QTextStream::UnicodeUTF8);

    HeadlineList headlines;

    QString line = stream.readLine();
    while(!line.isNull()) {
        headlines[line] = true;

        line = stream.readLine();
    }

    return headlines;
}

void RssPlugin::setOldHeadlines(HeadlineList& headlines, const QString& newsFeed)
{
    QString fileName = locateLocal("appdata", "rss/"+newsFeed+".read");
    QFile file(fileName);
    file.open(IO_WriteOnly);

    QTextStream stream(&file);
    stream.setEncoding(QTextStream::UnicodeUTF8);

    QStringList headlineList = headlines.keys();
    for(QStringList::Iterator it = headlineList.begin(); it != headlineList.end(); ++it) {
        stream << (*it) << endl;
    }
}

};
