/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>

#include <kapp.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdialog.h>
#include <kiconloader.h>
#include <klibloader.h>
#include <klocale.h>
#include <ktrader.h>

#include "pluginsettingpage.h"

namespace EventWatcher {

PluginSettingPage::PluginSettingPage(QWidget *parent, const char *name)
 : QWidget(parent, name),
   m_listView(0)
{
    QHBoxLayout *l1 = new QHBoxLayout(this);
    m_listView = new PluginListView(this);
    new PluginListViewToolTip(m_listView->viewport(), m_listView); // set tooltip which displays some plugin info...
    l1->addWidget(m_listView);
    l1->addSpacing( KDialog::spacingHint() );
    m_config = new PluginConfigDialog(this);
    m_config->hide();
    l1->addWidget(m_config);
    l1->setStretchFactor(m_listView, 1);
    l1->setStretchFactor(m_config, 3);

//     pluginSelectionChanged(); // update info & config dialog content

    connect( m_listView, SIGNAL(stateChange(PluginListItem*, bool)), this, SIGNAL(changed()) );
    connect( m_listView, SIGNAL(selectionChanged()), this, SLOT(pluginSelectionChanged()) );
    connect( m_listView, SIGNAL(stateChange(PluginListItem*, bool)), this, SLOT(pluginStateChange(PluginListItem*, bool)) );
    connect( m_config, SIGNAL(changed()), this, SIGNAL(changed()) );
}


PluginSettingPage::~PluginSettingPage()
{
}

void PluginSettingPage::pluginSelectionChanged()
{
    if( m_listView->currentItem() && static_cast<PluginListItem*>(m_listView->currentItem())->info() ) {
        KService::Ptr service = static_cast<PluginListItem*>(m_listView->currentItem())->info()->service;
        m_config->setPluginService( service );
        m_config->setEnabled( static_cast<PluginListItem*>(m_listView->currentItem())->isOn() );
    }
}

void PluginSettingPage::pluginStateChange(PluginListItem*, bool enabled)
{
    m_config->setEnabled(enabled);
}

void PluginSettingPage::slotApply()
{
    m_listView->updateConfig();
    m_config->save();
}

void PluginSettingPage::slotDefaults()
{
    m_config->defaults();
    // TODO: set default plugin enabled state too...
}

PluginListItem::PluginListItem(PluginSettingInfo* info, QListView *parent)
 : QCheckListItem(parent, info->service->name(), CheckBox),
   m_info(info),
   silentStateChange(false)
{
    silentStateChange = true; // don't emit a signal when we set the initial state
    setOn(m_info->enabled);
    silentStateChange = false;

    // set icon...
    QString icon = info->service->icon();
    if(!icon.isEmpty()) {
        setPixmap(0, SmallIcon( icon, IconSize( KIcon::Small ) ) );
    }

    // set tooltip with some plugin info...
//     QString tip;
//     tip += i18n("<b>Description:</b> %1\n").arg(info->service->comment());
//     QToolTip::add( this, tip );
}

void PluginListItem::stateChange(bool b)
{
    if(!silentStateChange) {
        m_info->enabled = b;
        static_cast<PluginListView *>(listView())->stateChanged(this, b);
    }
}

PluginListView::PluginListView(QWidget* parent, const char *name)
 : KListView(parent, name)
{
    addColumn( i18n("Name") );
    setResizeMode( AllColumns );

    updateList();
}

PluginListView::~PluginListView()
{
}

void PluginListView::updateList()
{
    fetchPluginList();
    readConfig();

    clear();
    for(PluginSettingInfoList::Iterator it(m_pluginList.begin()); it != m_pluginList.end(); ++it) {
        insertItem( new PluginListItem(&(*it), this) );
    }
}

void PluginListView::updateConfig()
{
    writeConfig();
}

void PluginListView::fetchPluginList()
{
    m_pluginList.clear();

    KTrader::OfferList traderResult = KTrader::self()->query("EventWatcher/Plugin");
    for(KTrader::OfferListIterator it(traderResult.begin()); it != traderResult.end(); ++it) {
        PluginSettingInfo p;
        p.service = *it;
        p.enabled = false;

        m_pluginList.append(p);
    }
}

void PluginListView::readConfig()
{
    KConfig *config = kapp->config();
    config->setGroup("Plugins");
    for(PluginSettingInfoList::Iterator it(m_pluginList.begin()); it != m_pluginList.end(); ++it) {
        (*it).enabled = config->readBoolEntry( (*it).service->library()+"Enabled",
                                               (*it).service->property("X-KDE-PluginInfo-EnabledByDefault").toBool() );
    }
}

void PluginListView::writeConfig()
{
    KConfig *config = kapp->config();
    config->setGroup("Plugins");
    for(PluginSettingInfoList::Iterator it(m_pluginList.begin()); it != m_pluginList.end(); ++it) {
//         kdDebug() << "writeEntry: " << (*it).service->library() << " " << (*it).enabled << endl;
        config->writeEntry( (*it).service->library()+"Enabled", (*it).enabled );
    }
}

void PluginListView::stateChanged(PluginListItem *item, bool b)
{
  emit stateChange(item, b);
}

PluginListViewToolTip::PluginListViewToolTip( QWidget *parent, KListView *lv )
 : QToolTip( parent ),
   m_listView( lv )
{
}

void PluginListViewToolTip::maybeTip( const QPoint &pos )
{
    if ( !parentWidget() || !m_listView )
        return;

    PluginListItem *item = dynamic_cast<PluginListItem *>( m_listView->itemAt( pos ) );
    if ( !item )
        return;

    QString toolTip;
    if(!item->info()->service->comment().isEmpty())
        toolTip += i18n("<b>Description:</b> %1<br>")
            .arg(item->info()->service->comment());
    if(!item->info()->service->property("X-KDE-PluginInfo-Author").toString().isEmpty())
        toolTip += i18n("<b>Author:</b> %1<br>")
            .arg(item->info()->service->property("X-KDE-PluginInfo-Author").toString());
    if(!item->info()->service->property("X-KDE-PluginInfo-Email").toString().isEmpty())
        toolTip += i18n("<b>Email:</b> %1<br>")
            .arg(item->info()->service->property("X-KDE-PluginInfo-Email").toString());
    if(!item->info()->service->property("X-KDE-PluginInfo-Version").toString().isEmpty())
        toolTip += i18n("<b>Version:</b> %1<br>")
            .arg(item->info()->service->property("X-KDE-PluginInfo-Version").toString());
    if(!item->info()->service->property("X-KDE-PluginInfo-License").toString().isEmpty())
        toolTip += i18n("<b>License:</b> %1")
            .arg(item->info()->service->property("X-KDE-PluginInfo-License").toString());

    tip( m_listView->itemRect( item ), toolTip );
}

PluginConfigDialog::PluginConfigDialog(QWidget *parent, const char *name)
 : QVGroupBox( i18n("Plugin Options"), parent, name ),
   m_configObject(0),
   m_oldLibName(0)
{
    setFlat(true);
}

PluginConfigDialog::~PluginConfigDialog()
{
    delete m_configObject;
    m_configObject = 0;

    if( !m_oldLibName.isNull() )
        KLibLoader::self()->unloadLibrary( m_oldLibName );
}

void PluginConfigDialog::setPluginService(const KService::Ptr& service)
{
    delete m_configObject;
    m_configObject = 0;

    KLibLoader* loader = KLibLoader::self();

    if( !m_oldLibName.isNull() )
        loader->unloadLibrary( m_oldLibName );

    QString libName = service->library()+"_config";

    KLibrary* library = loader->library( libName );
    if( library != 0 ) {
        void* alloc_ptr = library->symbol("allocate_config");
        if( alloc_ptr != 0 ) {
            QObject* (*allocatePlugin)( QWidget* parent ) = (QObject* (*)(QWidget* parent))alloc_ptr;
            m_configObject = (QObject*)(allocatePlugin( this ));
            connect(m_configObject, SIGNAL(changed()), this, SIGNAL(changed()) );
            connect(this, SIGNAL(signalSave()), m_configObject, SLOT(save()) );
            connect(this, SIGNAL(signalDefaults()), m_configObject, SLOT(defaults()) );

            m_oldLibName = libName;
            this->show();
            return;
        }
    }

    m_oldLibName = QString::null;
    this->hide();
}

void PluginConfigDialog::save()
{
    emit( signalSave() );
}

void PluginConfigDialog::defaults()
{
    emit( signalDefaults() );
}

};
#include "pluginsettingpage.moc"
