/***************************************************************************
 *   Copyright (C) 2004 by Sandro Giessl                                   *
 *   ceebx@users.sourceforge.net                                           *
 *                                                                         *
 *   based on KopeteBalloon:                                               *
 *   Copyright (c) 2002      by Duncan Mac-Vicar Prett <duncan@kde.org>    *
 *   Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org> *
 *   Portions of this code based on Kim Applet code                        *
 *   Copyright (c) 2000-2002 by Malte Starostik        <malte@kde.org>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qpointarray.h>
#include <qregion.h>
#include <qtimer.h>
#include <qtooltip.h>

#include <kdialog.h>
#include <kglobal.h>
#include <klocale.h>
// #include <netwm.h>
#include <kglobalsettings.h>

#include "config.h"
#include "popupmanager.h"
#include "lib/eventdata.h"
#include "eventcontainer.h"
#include "mainwidget.h"

namespace EventWatcher {

static const int POPUP_FLAGS = Qt::WStyle_Customize | Qt::WDestructiveClose | Qt::WX11BypassWM
                             | Qt::WStyle_StaysOnTop | Qt::WStyle_Tool | Qt::WStyle_NoBorder;

PopupManager::PopupManager(MainWidget *parent)
 : QWidget(0, "popup_manager", POPUP_FLAGS),m_parent(parent)
{
    hide();

    // construct popup layout
    QVBoxLayout *mainLayout = new QVBoxLayout(this, 12, KDialog::spacingHint());

    QVBoxLayout *l1 = new QVBoxLayout(mainLayout, KDialog::spacingHint(), "Layout1");
    m_label = new QLabel(this, "m_label");
    m_label->setAlignment(AlignAuto|AlignCenter|ExpandTabs);
    l1->addWidget(m_label);
    m_shortTitleLabel = new QLabel(this, "m_shorTitleLabel");
    m_shortTitleLabel->setAlignment(AlignLeft);
    m_shortTitleLabel->setFrameStyle( QFrame::LineEditPanel | QFrame::Sunken );
    m_shortTitleLabel->setShown( Config::popupShowDetails() );
    l1->addWidget(m_shortTitleLabel);

    QHBoxLayout *l2 = new QHBoxLayout(mainLayout, KDialog::spacingHint(), "Layout2");
    m_viewButton = new QPushButton(i18n("to view", "View"), this, "m_viewButton");
    connect(m_viewButton, SIGNAL(clicked()), this, SIGNAL(restoreMainWindow()) );
    m_detailsButton = new QPushButton(i18n("Details"), this, "m_detailsButton");
    m_detailsButton->setToggleButton(true);
    m_detailsButton->setOn( Config::popupShowDetails() );
    connect(m_detailsButton, SIGNAL(toggled(bool)), this, SLOT(slotSetShowDetails(bool)));
    l2->addStretch();
    l2->addWidget(m_viewButton);
    l2->addWidget(m_detailsButton);
    l2->addStretch();

    setPalette(QToolTip::palette());
    setAutoMask(TRUE);

    // setup the closeTimer...
    m_closeTimer = new QTimer(this);
    connect( m_closeTimer, SIGNAL(timeout()), this, SLOT(slotClose()) );

    // use the update timer to group all events created within a specific time into one popup.
    m_updateTimer = new QTimer(this);
    connect( m_updateTimer, SIGNAL(timeout()), this, SLOT(slotUpdateTimerTimeout()) );
}


PopupManager::~PopupManager()
{
}

void PopupManager::setAnchor(const QPoint &anchor)
{
    m_anchor = anchor;
    updateMask();
}

void PopupManager::slotClose()
{
    hide();
}

void PopupManager::slotEventAdded(EventItem* event)
{
    // don't create the popup immedialety but start a 1.5 second-timer everytime a new
    // event is added. This has the effect that events will be grouped together...
    QString eventShortTitle = event->data.shortTitle();
    if( !m_eventBuffer.contains(eventShortTitle) )
        m_eventBuffer[eventShortTitle] = 0;
    m_eventBuffer[eventShortTitle] += 1;
    m_updateTimer->start( 1500, true );
}

void PopupManager::slotUpdateTimerTimeout()
{
    bool leave = false;
    if( ! Config::popupEnabled() ) leave = true;
    if(m_parent->isVisible()) leave = true; // don't show a popup when the main window is visible
    if(m_eventBuffer.count() <= 0) leave = true; // don't show a popup when there aren't any new events.
    if( leave ) {
        // return, but also clear the m_eventBuffer!
        m_eventBuffer.clear();
        return;
    }

    int count = 0;
    QString shortTitles;
    for(EventPopupBuffer::iterator it = m_eventBuffer.begin(); it != m_eventBuffer.end(); ++it) {
        int n = (*it);
        if( n > 1 ) {
            shortTitles += QString("%1 (%2)\n").arg(it.key()).arg(n);
        } else {
            shortTitles += it.key() + "\n";
        }
        count += n; // count the number of events...
    }
    m_shortTitleLabel->setText( shortTitles );
    m_eventBuffer.clear();

    m_label->setText(i18n("<b>%1</b> new events").arg(count));

    slotSetShowDetails( Config::popupShowDetails() );

    // close the poup in 5 seconds... maybe I should make this configurable?
    m_closeTimer->start( Config::popupCloseDelay()*1000, true);

    layout()->activate();
    resize(sizeHint()); // resize the popup...
    show();
    updateMask();
}

void PopupManager::slotSetShowDetails(bool show)
{
    m_shortTitleLabel->setShown( show );
    m_detailsButton->setOn( Config::popupShowDetails() );
    Config::setPopupShowDetails(show);
    Config::writeConfig();

    // restart the timer...
    if( m_closeTimer->isActive() )
        m_closeTimer->start(Config::popupCloseDelay()*1000, true);

    layout()->activate();
    resize(sizeHint()); // resize the popup...
    updateMask();
}

void PopupManager::updateMask()
{
    QRegion mask(5, 5, width() - 10, height() - 10);

    QPoint corners[8] = {
        QPoint(width() - 25, 5),
        QPoint(5, 5),
        QPoint(5, height() - 25),
        QPoint(width() - 25, height() - 25),
        QPoint(width() - 5, 5),
        QPoint(5, 5),
        QPoint(5, height() - 5),
        QPoint(width() - 5, height() - 5)
    };

    for (int i = 0; i < 4; ++i) {
        QPointArray corner;
        corner.makeArc(corners[i].x(), corners[i].y(), 20, 20,
                       i * 16 * 90, 16 * 90);
        corner.resize(corner.size() + 1);
        corner.setPoint(corner.size() - 1, corners[i + 4]);
        mask -= corner;
    }

    // get screen-geometry for screen our anchor is on
    // (geometry can differ from screen to screen!
    QRect deskRect = KGlobalSettings::desktopGeometry(m_anchor);

    bool bottom = (m_anchor.y() + height()) > (deskRect.height() - 48);
    bool right = (m_anchor.x() + width()) > (deskRect.width() - 48);

    QPointArray arrow(4);
    arrow.setPoint(0, QPoint(right ? width() : 0, bottom ? height() : 0));
    arrow.setPoint(1, QPoint(right ? width() - 5 : 5,
                   bottom ? height() - 15 : 15));
    arrow.setPoint(2, QPoint(right ? width() - 15 : 15,
                   bottom ? height() - 5 : 5));
    arrow.setPoint(3, arrow[0]);
    mask += arrow;
    setMask(mask);

    move(right ? m_anchor.x() - width() : m_anchor.x(),
         bottom ? m_anchor.y() - height() : m_anchor.y());
}

void PopupManager::mouseReleaseEvent( QMouseEvent * )
{
    // close the popup on mouse click (->release).
    slotClose();
}

};

#include "popupmanager.moc"
